//
// Created by Ari on 10/1/24.
//
#include <iostream>
using namespace std;

#ifndef HW3_STAFF_TTT_H
#define HW3_STAFF_TTT_H

// various types.
using PLAYER_CODE = string; // represents the code of a player
using GAME_CODE = string;   // represents the code of a game
using SQUARE = char;        // 'x', 'y', '-' or '*'

// PLAYER functions
/**
 * Creates a new player with name [name].
 * @param name The name of the new player.
 * @return A code for the new player.
 */
PLAYER_CODE newPlayer(string name);

/**
 * @param thePlayerCode The code for the player in question.
 * @return The name associated with code [thePlayerCode].
 */
string getName(PLAYER_CODE thePlayerCode);

/**
 * @param thePlayerCode The code for the player in question.
 * @return Status information about the player with code [thePlayerCode] in JSON format.
 * @see https://en.wikipedia.org/wiki/JSON - a description of JSON
 * @see https://www.w3schools.com/whatis/whatis_json.asp - descriptions through examples
 */
string getPlayerStatus(PLAYER_CODE thePlayerCode);

/**
 * @return A human readable list of the top players in the system.
 */
string getTopRatings();




// GAME FUNCTIONS
/**
 * Creates a new game.
 * @param xPlayerCode The code of the player playing for 'x'.
 * @param yPlayerCode The code of the player playing for 'y'.
 * @return a code for the new game.
 */
GAME_CODE newGame(PLAYER_CODE xPlayerCode, PLAYER_CODE yPlayerCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return The side that is currently set to play in the game with code [theGameCode].
 */
SQUARE getWhoToPlay(GAME_CODE theGameCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return How many of the same-type square in a row are needed to win.
 */
int getInARow(GAME_CODE theGameCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return The date-time when the game with code [theGameCode] was last modified.
 */
string getModTime(GAME_CODE theGameCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return The current status of the game with code [theGameCode] in JSON format.
 * @see https://en.wikipedia.org/wiki/JSON - a description of JSON
 * @see https://www.w3schools.com/whatis/whatis_json.asp - descriptions through examples
 */
string getGameStatus(GAME_CODE theGameCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return The number of rows on the board of the current game.
 */
int getRows(GAME_CODE theGameCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return The number of columns on the board of the game with the given game code.
 */
int getCols(GAME_CODE theGameCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return The next player to move in the game with the given game code.
 */
SQUARE getToMove(GAME_CODE theGameCode);

/**
 * @param theGameCode The code of the Game in question.
 * @return A string representing the current board in the game with the given game code.
 */
string getBoard(GAME_CODE theGameCode);

/**
 * @param board a game board produced by the function getBoard(GAME_CODE theGameCode).
 * @param theRow a row on the game board.
 * @param theCol a colum on the game board.
 * @return The square at row [theRow] and column [theCol] on the given board.
 */
SQUARE getBoardSquare(string board, int theRow, int theCol);

/**
 * @param theGameCode The code of the Game in question.
 * @return Who has won the game, or '-' if no one has won yet.
 */
SQUARE getHasWon(GAME_CODE theGameCode);

/**
 * Makes a move for side [side] by the player with code [thePlayerCode]
 * at row [row] and column [col] of the board for the game with
 * code [theGameCode].
 * @param theGameCode The code of the game being played.
 * @param thePlayerCode The code of the player making the move.
 * @param theSize The side that is playing.
 * @param row The row at which the move is made, starting at 0.
 * @param col The column at which the move is made, starting at 0.
 * @return a description of an error, if an error occurred (and an empty string otherwise).
 *   Errors include playing out of turn, playing for the wrong side,
 *   or playing to an occupied square, among others.
 */
string makeMove(GAME_CODE theGameCode,
                PLAYER_CODE thePlayerCode,
                SQUARE theSide,
                int row,
                int col);

#endif //HW3_STAFF_TTT_H
